@@ -23,12 +23,14 @@ import (
 	"github.com/gcla/termshark/v2/cli"
 	"github.com/gcla/termshark/v2/convs"
 	"github.com/gcla/termshark/v2/pcap"
+	"github.com/gcla/termshark/v2/shark"
 	"github.com/gcla/termshark/v2/streams"
 	"github.com/gcla/termshark/v2/system"
 	"github.com/gcla/termshark/v2/theme"
 	"github.com/gcla/termshark/v2/tty"
 	"github.com/gcla/termshark/v2/ui"
 	"github.com/gcla/termshark/v2/widgets/filter"
+	"github.com/gcla/termshark/v2/widgets/wormhole"
 	"github.com/gdamore/tcell"
 	flags "github.com/jessevdk/go-flags"
 	"github.com/mattn/go-isatty"
@@ -58,9 +60,46 @@ func main() {
 	capinfo.Goroutinewg = &ensureGoroutinesStopWG
 	convs.Goroutinewg = &ensureGoroutinesStopWG
 	ui.Goroutinewg = &ensureGoroutinesStopWG
+	wormhole.Goroutinewg = &ensureGoroutinesStopWG
 
 	res := cmain()
 	ensureGoroutinesStopWG.Wait()
+
+	if !termshark.ShouldSwitchTerminal && !termshark.ShouldSwitchBack {
+		os.Exit(res)
+	}
+
+	os.Clearenv()
+	for _, e := range termshark.OriginalEnv {
+		ks := strings.SplitN(e, "=", 2)
+		if len(ks) == 2 {
+			os.Setenv(ks[0], ks[1])
+		}
+	}
+
+	exe, err := os.Executable()
+	if err != nil {
+		log.Warnf("Unexpected error determining termshark executable: %v", err)
+		os.Exit(1)
+	}
+
+	switch {
+	case termshark.ShouldSwitchTerminal:
+		os.Setenv("TERMSHARK_ORIGINAL_TERM", os.Getenv("TERM"))
+		os.Setenv("TERM", fmt.Sprintf("%s-256color", os.Getenv("TERM")))
+	case termshark.ShouldSwitchBack:
+		os.Setenv("TERM", os.Getenv("TERMSHARK_ORIGINAL_TERM"))
+		os.Setenv("TERMSHARK_ORIGINAL_TERM", "")
+	}
+
+	// Need exec because we really need to re-initialize everything, including have
+	// all init() functions be called again
+	err = syscall.Exec(exe, os.Args, os.Environ())
+	if err != nil {
+		log.Warnf("Unexpected error exec-ing termshark %s: %v", exe, err)
+		res = 1
+	}
+
 	os.Exit(res)
 }
 
@@ -68,6 +107,9 @@ func cmain() int {
 	startedSuccessfully := false // true if we reached the point where packets were received and the UI started.
 	uiSuspended := false         // true if the UI was suspended due to SIGTSTP
 
+	// Preserve in case we need to re-exec e.g. if the user switches TERM
+	termshark.OriginalEnv = os.Environ()
+
 	sigChan := make(chan os.Signal, 100)
 	// SIGINT and SIGQUIT will arrive only via an external kill command,
 	// not the keyboard, because our line discipline is set up to pass
@@ -96,16 +138,16 @@ func cmain() int {
 	stdConf := configdir.New("", "termshark")
 	dirs := stdConf.QueryFolders(configdir.Cache)
 	if err := dirs[0].CreateParentDir("dummy"); err != nil {
-		fmt.Printf("Warning: could not create cache dir: %v\n", err)
+		fmt.Fprintf(os.Stderr, "Warning: could not create cache dir: %v\n", err)
 	}
 	dirs = stdConf.QueryFolders(configdir.Global)
 	if err := dirs[0].CreateParentDir("dummy"); err != nil {
-		fmt.Printf("Warning: could not create config dir: %v\n", err)
+		fmt.Fprintf(os.Stderr, "Warning: could not create config dir: %v\n", err)
 	}
 	viper.AddConfigPath(dirs[0].Path)
 
 	if f, err := os.OpenFile(filepath.Join(dirs[0].Path, "termshark.toml"), os.O_RDONLY|os.O_CREATE, 0666); err != nil {
-		fmt.Printf("Warning: could not create initial config file: %v\n", err)
+		fmt.Fprintf(os.Stderr, "Warning: could not create initial config file: %v\n", err)
 	} else {
 		f.Close()
 	}
@@ -147,6 +189,8 @@ func cmain() int {
 		}
 	}
 
+	// On Windows, termshark itself is used to tail the pcap generated by dumpcap, and the output
+	// is fed into tshark -T psml ...
 	if tsopts.TailFileValue() != "" {
 		err = termshark.TailFile(tsopts.TailFileValue())
 		if err != nil {
@@ -270,7 +314,7 @@ func cmain() int {
 	// terminal emumlator supports 256 colors.
 	termVar := termshark.ConfString("main.term", "")
 	if termVar != "" {
-		log.Infof("Configuration file overrides TERM setting, using TERM=%s", termVar)
+		fmt.Fprintf(os.Stderr, "Configuration file overrides TERM setting, using TERM=%s\n", termVar)
 		os.Setenv("TERM", termVar)
 	}
 
@@ -418,6 +462,7 @@ func cmain() int {
 
 	// go-flags returns [""] when no extra args are provided, so I can't just
 	// test the length of this slice
+	termshark.ReverseStringSlice(filterArgs)
 	argsFilter := strings.Join(filterArgs, " ")
 
 	// Work out capture filter afterwards because we need to determine first
@@ -434,6 +479,25 @@ func cmain() int {
 		captureFilter = argsFilter
 	}
 
+	// -w something
+	if opts.WriteTo != "" {
+		if len(fileSrcs) > 0 {
+			fmt.Fprintf(os.Stderr, "The -w flag is incompatible with regular capture sources %v\n", fileSrcs)
+			return 1
+		}
+		if opts.WriteTo == "-" {
+			fmt.Fprintf(os.Stderr, "Cannot set -w to stdout. Target file must be regular or a symlink.\n")
+			return 1
+		}
+		// If the file does not exist, then proceed. If it does exist, check it is something "normal".
+		if _, err = os.Stat(string(opts.WriteTo)); err == nil || !os.IsNotExist(err) {
+			if !system.FileRegularOrLink(string(opts.WriteTo)) {
+				fmt.Fprintf(os.Stderr, "Cannot set -w to %s. Target file must be regular or a symlink.\n", opts.WriteTo)
+				return 1
+			}
+		}
+	}
+
 	displayFilter := opts.DisplayFilter
 
 	// Validate supplied filters e.g. no capture filter when reading from file
@@ -482,7 +546,7 @@ func cmain() int {
 		}()
 	}
 
-	for _, dir := range []string{termshark.CacheDir(), termshark.PcapDir()} {
+	for _, dir := range []string{termshark.CacheDir(), termshark.DefaultPcapDir(), termshark.PcapDir()} {
 		if _, err = os.Stat(dir); os.IsNotExist(err) {
 			err = os.Mkdir(dir, 0777)
 			if err != nil {
@@ -640,8 +704,8 @@ func cmain() int {
 		// fifo. In all cases, we save the packets to a file so that if a
 		// filter is applied, we can restart - and so that we preserve the
 		// capture at the end of running termshark.
-		if len(pcap.FileSystemSources(psrcs)) == 0 && startedSuccessfully {
-			fmt.Printf("Packets read from %s have been saved in %s\n", pcap.SourcesString(psrcs), ifacePcapFilename)
+		if len(pcap.FileSystemSources(psrcs)) == 0 && startedSuccessfully && !ui.WriteToSelected && !ui.WriteToDeleted {
+			fmt.Fprintf(os.Stderr, "Packets read from %s have been saved in %s\n", pcap.SourcesString(psrcs), ifacePcapFilename)
 		}
 	}()
 
@@ -693,15 +757,21 @@ func cmain() int {
 	}
 
 	var ifaceTmpFile string
+	var waitingForPackets bool
 
+	// no file sources - so interface or fifo
 	if len(pcap.FileSystemSources(psrcs)) == 0 {
-		srcNames := make([]string, 0, len(psrcs))
-		for _, psrc := range psrcs {
-			srcNames = append(srcNames, psrc.Name())
+		if opts.WriteTo != "" {
+			ifaceTmpFile = string(opts.WriteTo)
+			ui.WriteToSelected = true
+		} else {
+			srcNames := make([]string, 0, len(psrcs))
+			for _, psrc := range psrcs {
+				srcNames = append(srcNames, psrc.Name())
+			}
+			ifaceTmpFile = pcap.TempPcapFile(srcNames...)
 		}
-		ifaceTmpFile = pcap.TempPcapFile(srcNames...)
-
-		fmt.Printf("(The termshark UI will start when packets are detected...)\n")
+		waitingForPackets = true
 	} else {
 		// Start UI right away, reading from a file
 		close(ui.StartUIChan)
@@ -799,6 +869,24 @@ func cmain() int {
 		},
 	}
 
+	// Do this before the load starts, so that the PSML process has a guaranteed safe
+	// PSML column format to use when it begins. The call to RequestLoadPcapWithCheck,
+	// for example, will access the setting for preferred PSML columns.
+	//
+	// Init a global variable with the list of all valid tshark columns and
+	// their formats. This might start a tshark process if the data isn't
+	// cached. If so, print a message to console - "initializing". I'm not doing
+	// anything smarter or async - it's not worth it, this should take a fraction
+	// of a second.
+	err = shark.InitValidColumns()
+
+	// If this message is needed, we want it to appear after the init message for the packet
+	// columns - after InitValidColumns
+	if waitingForPackets {
+		fmt.Fprintf(os.Stderr, fmt.Sprintf("(The termshark UI will start when packets are detected on %s...)\n",
+			strings.Join(pcap.SourcesNames(psrcs), " or ")))
+	}
+
 	// Refresh
 	fileSrcs = pcap.FileSystemSources(psrcs)
 	if len(fileSrcs) > 0 {
@@ -813,7 +901,7 @@ func cmain() int {
 			app.Run(gowid.RunFunction(func(app gowid.IApp) {
 				ui.FilterWidget.SetValue(displayFilter, app)
 			}))
-			ui.RequestLoadPcapWithCheck(absfile, displayFilter, ui.NoGlobalJump, app)
+			ui.RequestLoadPcap(absfile, displayFilter, ui.NoGlobalJump, app)
 		}
 		validator.Valid = &filter.ValidateCB{Fn: doit, App: app}
 		validator.Validate(displayFilter)
@@ -872,16 +960,21 @@ func cmain() int {
 		ui.Loader.CloseMain()
 	}
 
-	inactiveDuration := 30 * time.Second
+	inactiveDuration := 60 * time.Second
 	inactivityTimer := time.NewTimer(inactiveDuration)
 
 	var progCancelTimer *time.Timer
 
+	checkedPcapCache := false
+	checkPcapCacheDuration := 5 * time.Second
+	checkPcapCacheTimer := time.NewTimer(checkPcapCacheDuration)
+
 Loop:
 	for {
 		var finChan <-chan time.Time
 		var tickChan <-chan time.Time
 		var inactivityChan <-chan time.Time
+		var checkPcapCacheChan <-chan time.Time
 		var tcellEvents <-chan tcell.Event
 		var opsChan <-chan gowid.RunFunction
 		var afterRenderEvents <-chan gowid.IAfterRenderEvent
@@ -937,17 +1030,58 @@ Loop:
 		// Only display the progress bar if PSML is loading or if PDML is loading that is needed
 		// by the UI. If the PDML is an optimistic load out of the display, then no need for
 		// progress.
+		doprog := false
 		if ui.Loader.PsmlLoader.IsLoading() || (ui.Loader.PdmlLoader.IsLoading() && ui.Loader.PdmlLoader.LoadIsVisible()) {
+			if prevProgPercentage >= 1.0 {
+				if progCancelTimer != nil {
+					progCancelTimer.Reset(time.Duration(500) * time.Millisecond)
+					progCancelTimer = nil
+				}
+			} else {
+				ui.SetProgressWidget(app)
+				if progCancelTimer == nil {
+					progCancelTimer = time.AfterFunc(time.Duration(100000)*time.Hour, func() {
+						app.Run(gowid.RunFunction(func(app gowid.IApp) {
+							ui.ClearProgressWidget(app)
+							progCancelTimer = nil
+						}))
+					})
+				}
+			}
+
 			tickChan = progTicker.C // progress is only enabled when a pcap may be loading
-		} else {
-			// Reset for the next load
-			prevProgPercentage = 0.0
+
+			// Rule:
+			// - prefer progress if we can apply it to psml only (not pdml)
+			// - otherwise use a spinner if interface load or fifo load in operation
+			// - otherwise use progress for pdml
+			if system.HaveFdinfo {
+				// Prefer progress, if the OS supports it.
+				doprog = true
+				if ui.Loader.ReadingFromFifo() {
+					// But if we are have an interface load (or a pipe load), then we can't
+					// predict when the data will run out, so use a spinner. That's because we
+					// feed the data to tshark -T psml with a tail command which reads from
+					// the tmp file being created by the pipe/interface source.
+					doprog = false
+					if !ui.Loader.InterfaceLoader.IsLoading() && !ui.Loader.PsmlLoader.IsLoading() {
+						// Unless those loads are finished, and the only loading activity is now
+						// PDML/pcap, which is loaded on demand in blocks of 1000. Then we can
+						// use the progress bar.
+						doprog = true
+					}
+				}
+			}
 		}
 
 		if ui.Loader.InterfaceLoader.IsLoading() {
 			inactivityChan = inactivityTimer.C
 		}
 
+		if !checkedPcapCache {
+			checkPcapCacheChan = checkPcapCacheTimer.C
+		}
+
 		// Only process tcell and gowid events if the UI is running.
 		if ui.Running {
 			tcellEvents = app.TCellEvents
@@ -973,6 +1107,16 @@ Loop:
 
 		select {
 
+		case <-checkPcapCacheChan:
+			// Only check the cache dir if we own it; don't want to delete pcap files
+			// that might be shared with wireshark
+			if termshark.ConfBool("main.use-tshark-temp-for-pcap-cache", false) {
+				log.Infof("Termshark does not own the pcap temp dir %s; skipping size check", termshark.PcapDir())
+			} else {
+				termshark.PrunePcapCache()
+			}
+			checkedPcapCache = true
+
 		case <-inactivityChan:
 			ui.Fin.Activate()
 			app.Redraw()
@@ -993,8 +1137,9 @@ Loop:
 			// Need to do that here because the app won't know how many colors the screen
 			// has (and therefore which variant of the theme to load) until the screen is
 			// activated.
-			mode := theme.Mode(app.GetColorMode()).String() // more concise
-			themeName := termshark.ConfString(fmt.Sprintf("main.theme-%s", mode), "default")
+			mode := app.GetColorMode()
+			modeStr := theme.Mode(mode) // more concise
+			themeName := termshark.ConfString(fmt.Sprintf("main.theme-%s", modeStr), "default")
 			loaded := false
 			if themeName != "" {
 				err = theme.Load(themeName, app)
@@ -1027,6 +1172,33 @@ Loop:
 
 			ui.StartUIChan = nil // make sure it's not triggered again
 
+			if runtime.GOOS != "windows" {
+				if mode == gowid.Mode8Colors {
+					// If exists is true, it means we already tried and then reverted back, so
+					// just load up termshark normally with no further interruption.
+					if _, exists := os.LookupEnv("TERMSHARK_ORIGINAL_TERM"); !exists {
+						if !termshark.ConfBool("main.disable-term-helper", false) {
+							err = termshark.Does256ColorTermExist()
+							if err != nil {
+								log.Infof("Must use 8-color mode because 256-color version of TERM=%s unavailable - %v.", os.Getenv("TERM"), err)
+							} else {
+								time.AfterFunc(time.Duration(3)*time.Second, func() {
+									app.Run(gowid.RunFunction(func(app gowid.IApp) {
+										ui.SuggestSwitchingTerm(app)
+									}))
+								})
+							}
+						}
+					}
+				} else if os.Getenv("TERMSHARK_ORIGINAL_TERM") != "" {
+					time.AfterFunc(time.Duration(3)*time.Second, func() {
+						app.Run(gowid.RunFunction(func(app gowid.IApp) {
+							ui.IsTerminalLegible(app)
+						}))
+					})
+				}
+			}
+
 			defer func() {
 				// Do this to make sure the program quits quickly if quit is invoked
 				// mid-load. It's safe to call this if a pcap isn't being loaded.
@@ -1053,6 +1225,12 @@ Loop:
 				// will happen next time round because the quitRequested flag is checked.
 				stopLoaders()
 			}
+			if ui.CurrentWormholeWidget != nil {
+				ui.CurrentWormholeWidget.Close()
+			}
+			if ui.CurrentColsWidget != nil {
+				ui.CurrentColsWidget.Close()
+			}
 
 		case sig := <-sigChan:
 			if system.IsSigTSTP(sig) {
@@ -1121,40 +1299,6 @@ Loop:
 
 		case <-tickChan:
 			// We already know that we are LoadingPdml|LoadingPsml
-			ui.SetProgressWidget(app)
-			if progCancelTimer != nil {
-				progCancelTimer.Reset(time.Duration(500) * time.Millisecond)
-			} else {
-				progCancelTimer = time.AfterFunc(time.Duration(500)*time.Millisecond, func() {
-					app.Run(gowid.RunFunction(func(app gowid.IApp) {
-						ui.ClearProgressWidget(app)
-					}))
-				})
-			}
-
-			// Rule:
-			// - prefer progress if we can apply it to psml only (not pdml)
-			// - otherwise use a spinner if interface load or fifo load in operation
-			// - otherwise use progress for pdml
-			doprog := false
-			if system.HaveFdinfo {
-				// Prefer progress, if the OS supports it.
-				doprog = true
-				if ui.Loader.ReadingFromFifo() {
-					// But if we are have an interface load (or a pipe load), then we can't
-					// predict when the data will run out, so use a spinner. That's because we
-					// feed the data to tshark -T psml with a tail command which reads from
-					// the tmp file being created by the pipe/interface source.
-					doprog = false
-					if !ui.Loader.InterfaceLoader.IsLoading() && !ui.Loader.PsmlLoader.IsLoading() {
-						// Unless those loads are finished, and the only loading activity is now
-						// PDML/pcap, which is loaded on demand in blocks of 1000. Then we can
-						// use the progress bar.
-						doprog = true
-					}
-				}
-			}
-
 			if doprog {
 				app.Run(gowid.RunFunction(func(app gowid.IApp) {
 					prevProgPercentage = ui.UpdateProgressBarForFile(ui.Loader, prevProgPercentage, app)
@@ -1168,6 +1312,7 @@ Loop:
 		case ev := <-tcellEvents:
 			app.HandleTCellEvent(ev, gowid.IgnoreUnhandledInput)
 			inactivityTimer.Reset(inactiveDuration)
+			checkPcapCacheTimer.Reset(checkPcapCacheDuration)
 
 		case ev, ok := <-afterRenderEvents:
 			// This means app.Quit() has been called, which closes the AfterRenderEvents
