@@ -193,10 +193,12 @@ type PsmlLoader struct {
 	PsmlCmd IPcapCommand // gcla later todo - change to pid like PdmlPid
 
 	sync.Mutex
-	packetPsmlData    [][]string
-	packetPsmlColors  []PacketColors
-	packetPsmlHeaders []string
-	PacketNumberMap   map[int]int // map from actual packet row <section>12</section> to pos in unsorted table
+	packetAverageLength []averageTracker // length of num columns
+	packetMaxLength     []maxTracker     // length of num columns
+	packetPsmlData      [][]string
+	packetPsmlColors    []PacketColors
+	packetPsmlHeaders   []string
+	PacketNumberMap     map[int]int // map from actual packet row <section>12</section> to pos in unsorted table
 	// This would be affected by a display filter e.g. packet 12 might be the 1st packet in the table.
 	// I need this so that if the user jumps to a mark stored as "packet 12", I can find the right table row.
 	PacketCache *lru.Cache // i -> [pdml(i * 1000)..pdml(i+1*1000)] - accessed from any goroutine
@@ -313,16 +315,18 @@ func NewPcapLoader(cmds ILoaderCmds, runner IMainRunner, opts ...Options) *Paren
 
 func (c *ParentLoader) RenewPsmlLoader() {
 	c.PsmlLoader = &PsmlLoader{
-		PcapPsml:          c.PsmlLoader.PcapPsml,
-		tailCmd:           c.PsmlLoader.tailCmd,
-		PsmlCmd:           c.PsmlLoader.PsmlCmd,
-		packetPsmlData:    make([][]string, 0),
-		packetPsmlColors:  make([]PacketColors, 0),
-		packetPsmlHeaders: make([]string, 0, 10),
-		PacketNumberMap:   make(map[int]int),
-		startStage2Chan:   make(chan struct{}), // do this before signalling start
-		PsmlFinishedChan:  make(chan struct{}),
-		opt:               c.opt,
+		PcapPsml:            c.PsmlLoader.PcapPsml,
+		tailCmd:             c.PsmlLoader.tailCmd,
+		PsmlCmd:             c.PsmlLoader.PsmlCmd,
+		packetAverageLength: make([]averageTracker, 64),
+		packetMaxLength:     make([]maxTracker, 64),
+		packetPsmlData:      make([][]string, 0),
+		packetPsmlColors:    make([]PacketColors, 0),
+		packetPsmlHeaders:   make([]string, 0, 10),
+		PacketNumberMap:     make(map[int]int),
+		startStage2Chan:     make(chan struct{}), // do this before signalling start
+		PsmlFinishedChan:    make(chan struct{}),
+		opt:                 c.opt,
 	}
 	packetCache, err := lru.New(c.opt.CacheSize)
 	if err != nil {
@@ -371,8 +375,6 @@ func (p *ParentLoader) TurnOffPipe() {
 	}
 }
 
-var _ iPsmlLoaderEnv = (*ParentLoader)(nil)
-
 func (p *ParentLoader) PacketSources() []IPacketSource {
 	return p.psrcs
 }
@@ -425,35 +427,27 @@ func (c *ParentLoader) StopLoadPsmlAndIface(cb interface{}) {
 
 //======================================================================
 
-// NewFilter is essentially a completely new load - psml + pdml. But not iface, if that's running
-func (c *PacketLoader) NewFilter(newfilt string, cb interface{}, app gowid.IApp) {
+func (c *PacketLoader) Reload(filter string, cb interface{}, app gowid.IApp) {
+	c.stopTail()
+	c.stopLoadPsml()
+	c.stopLoadPdml()
 
-	log.Infof("Requested application of display filter '%v'", newfilt)
+	OpsChan <- gowid.RunFunction(func(app gowid.IApp) {
+		c.RenewPsmlLoader()
+		c.RenewPdmlLoader()
 
-	if c.DisplayFilter() == newfilt {
-		log.Infof("No operation - same filter applied ('%s').", newfilt)
-	} else {
-		c.stopTail()
-		c.stopLoadPsml()
-		c.stopLoadPdml()
+		// This is not ideal. I'm clearing the views, but I'm about to
+		// restart. It's not really a new source, so called the new source
+		// handler is an untify way of updating the current capture in the
+		// title bar again
+		handleClear(NoneCode, app, cb)
 
-		OpsChan <- gowid.RunFunction(func(app gowid.IApp) {
-			c.RenewPsmlLoader()
-			c.RenewPdmlLoader()
+		c.displayFilter = filter
 
-			// This is not ideal. I'm clearing the views, but I'm about to
-			// restart. It's not really a new source, so called the new source
-			// handler is an untify way of updating the current capture in the
-			// title bar again
-			handleClear(NoneCode, app, cb)
+		log.Infof("Applying display filter '%s'", filter)
 
-			c.displayFilter = newfilt
-
-			log.Infof("Applying new display filter '%s'", newfilt)
-
-			c.loadPsmlSync(c.InterfaceLoader, c, cb, app)
-		})
-	}
+		c.loadPsmlSync(c.InterfaceLoader, c, cb, app)
+	})
 }
 
 func (c *PacketLoader) LoadPcap(pcap string, displayFilter string, cb interface{}, app gowid.IApp) {
@@ -1015,8 +1009,17 @@ func (c *PdmlLoader) loadPcapSync(row int, visible bool, ps iPdmlLoaderEnv, cb i
 				// a message means the proc has started
 				// closed means it won't be started
 				// if closed, then pdmlCmd == nil
-				if (pdmlState == Terminated || (pdmlCancelledChan == nil && pdmlState == NotStarted)) &&
-					(pcapState == Terminated || (pcapCancelledChan == nil && pcapState == NotStarted)) {
+				// 04/11/21: I can't take a shortcut here and condition on Terminated || (cancelledChan == nil && NotStarted)
+				// See the pcap or pdml goroutines below. I block at the beginning, checking on the stage2 cancellation.
+				// If I get past that point, and there are no errors in the process invocation, I am guaranteed to start both
+				// the pdml and pcap processes. If there are errors, I am guaranteed to close the pcapPidChan with a defer.
+				// If I take a shortcut and end this goroutine via a stage2 cancellation before waiting for the pcap pid,
+				// then I'll block in that goroutine, trying to send to the pcapPidChan, but with nothing here to receive
+				// the value. In the pcap process goroutine, if I get past the stage2 cancellation check, then I need to
+				// have something to receive the pid - this goroutine. It needs to stay alive until it gets the pid, or a
+				// zero.
+				if (pdmlState == Terminated || (pdmlPidChan == nil && c.PdmlPid == 0)) &&
+					(pcapState == Terminated || (pcapPidChan == nil && c.PcapPid == 0)) {
 					// nothing to select on so break
 					break loop
 				}
@@ -1083,6 +1086,8 @@ func (c *PdmlLoader) loadPcapSync(row int, visible bool, ps iPdmlLoaderEnv, cb i
 				if err != nil {
 					if !issuedKill && unexpectedPdmlError(err) {
 						err = fmt.Errorf("Could not read PDML data: %v", err)
+						issuedKill = true
+						pdmlCancelFn()
 						HandleError(PdmlCode, app, err, cb)
 					}
 					if err == io.EOF {
@@ -1098,6 +1103,8 @@ func (c *PdmlLoader) loadPcapSync(row int, visible bool, ps iPdmlLoaderEnv, cb i
 						if err != nil {
 							if !issuedKill && unexpectedPdmlError(err) {
 								err = fmt.Errorf("Could not decode PDML data: %v", err)
+								issuedKill = true
+								pdmlCancelFn()
 								HandleError(PdmlCode, app, err, cb)
 							}
 							break Loop
@@ -1359,6 +1366,15 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 
 		//======================================================================
 
+		closedPipe := false
+		closePipe := func() {
+			if !closedPipe {
+				fifoPipeWriter.Close()
+				fifoPipeReader.Close()
+				closedPipe = true
+			}
+		}
+
 		if p.ReadingFromFifo() {
 			// PcapPsml will be nil if here
 
@@ -1378,8 +1394,7 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 			// is used as stdin for the psml command, which also runs in this
 			// goroutine.
 			defer func() {
-				fifoPipeWriter.Close()
-				fifoPipeReader.Close()
+				closePipe()
 			}()
 
 			// wrap the read end of the pipe with a Read() function that counts
@@ -1425,6 +1440,10 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 				if err != nil {
 					log.Infof("Did not kill tshark psml process: %v", err)
 				}
+
+				if p.ReadingFromFifo() {
+					closePipe()
+				}
 			}
 
 		loop:
@@ -1464,7 +1483,7 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 					}
 				}
 
-				if state == Terminated || (intCancelledChan == nil && state == NotStarted) {
+				if state == Terminated || (pidChan == nil && state == NotStarted) {
 					break loop
 				}
 			}
@@ -1590,7 +1609,7 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 
 					// successfully started then died/kill, OR
 					// was never started, won't be started, and cancelled
-					if state == Terminated || (cancelledChan == nil && state == NotStarted) {
+					if state == Terminated || (pidChan == nil && state == NotStarted) {
 						break loop
 					}
 				}
@@ -1717,6 +1736,7 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 		// </packet>
 
 		var curPsml []string
+		var curCounts []int
 		var fg string
 		var bg string
 		var pidx int
@@ -1741,9 +1761,14 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 				switch tok.Name.Local {
 				case "structure":
 					structure = false
+					p.Lock()
+					// Don't keep the first column - we add on column number to all PSML
+					// loads whether or not the user wants it to track table number -> packet
+					// number. This is then stripped from the columns shown to the user.
+					p.packetPsmlHeaders = p.packetPsmlHeaders[1:]
+					p.Unlock()
 				case "packet":
 					p.Lock()
-					p.packetPsmlData = append(p.packetPsmlData, curPsml)
 
 					// Track the mapping of packet number <section>12</section> to position
 					// in the table e.g. 5th element. This is so that I can jump to the correct
@@ -1752,7 +1777,22 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 					if err != nil {
 						log.Fatal(err)
 					}
-					p.PacketNumberMap[pidx] = len(p.packetPsmlData) - 1
+					p.PacketNumberMap[pidx] = len(p.packetPsmlData)
+
+					p.packetPsmlData = append(p.packetPsmlData, curPsml[1:])
+
+					if len(p.packetAverageLength) > len(curPsml)-1 {
+						p.packetAverageLength = p.packetAverageLength[0 : len(curPsml)-1]
+					}
+					if len(p.packetMaxLength) > len(curPsml)-1 {
+						p.packetMaxLength = p.packetMaxLength[0 : len(curPsml)-1]
+					}
+
+					for i, ct := range curCounts[1:] {
+						// skip the first one - that's not displayed in the UI. We always have element 0 as No.
+						p.packetAverageLength[i].update(ct)
+						p.packetMaxLength[i].update(ct)
+					}
 
 					p.packetPsmlColors = append(p.packetPsmlColors, PacketColors{
 						FG: psmlColorToIColor(fg),
@@ -1764,6 +1804,7 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 					ready = false
 					// Means we got </section> without any char data i.e. empty <section>
 					if empty {
+						curCounts = append(curCounts, 0)
 						curPsml = append(curPsml, "")
 					}
 				}
@@ -1773,6 +1814,7 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 					structure = true
 				case "packet":
 					curPsml = make([]string, 0, 10)
+					curCounts = make([]int, 0, 10)
 					fg = ""
 					bg = ""
 					for _, attr := range tok.Attr {
@@ -1798,6 +1840,7 @@ func (p *PsmlLoader) loadPsmlSync(iloader *InterfaceLoader, e iPsmlLoaderEnv, cb
 						}))
 					} else {
 						curPsml = append(curPsml, string(format.TranslateHexCodes(tok)))
+						curCounts = append(curCounts, len(curPsml[len(curPsml)-1]))
 						empty = false
 					}
 				}
@@ -1833,10 +1876,6 @@ func (c *PsmlLoader) PacketCacheFn() *lru.Cache { // i -> [pdml(i * 1000)..pdml(
 	return c.PacketCache
 }
 
-func (c *PsmlLoader) packetPsmlDataFn() [][]string {
-	return c.packetPsmlData
-}
-
 // Assumes this is a clean stop, not an error
 func (p *ParentLoader) stopTail() {
 	p.tailStoppedDeliberately = true
@@ -1870,6 +1909,22 @@ func (p *PsmlLoader) PsmlColors() []PacketColors {
 	return p.packetPsmlColors
 }
 
+func (p *PsmlLoader) PsmlAverageLengths() []gwutil.IntOption {
+	res := make([]gwutil.IntOption, 0, len(p.packetAverageLength))
+	for _, avg := range p.packetAverageLength {
+		res = append(res, avg.average())
+	}
+	return res
+}
+
+func (p *PsmlLoader) PsmlMaxLengths() []int {
+	res := make([]int, 0, len(p.packetMaxLength))
+	for _, maxer := range p.packetMaxLength {
+		res = append(res, int(maxer.max()))
+	}
+	return res
+}
+
 // if done==true, then this cache entry is complete
 func (p *PsmlLoader) updateCacheEntryWithPdml(row int, pdml []IPdmlPacket, done bool) {
 	var ce CacheEntry
@@ -2082,7 +2137,7 @@ func (i *InterfaceLoader) loadIfacesSync(e iIfaceLoaderEnv, cb interface{}, app
 			// a message means the proc has started
 			// closed means it won't be started
 			// if closed, then pdmlCmd == nil
-			if state == Terminated || (cancelledChan == nil && state == NotStarted) {
+			if state == Terminated || (pidChan == nil && state == NotStarted) {
 				// nothing to select on so break
 				break loop
 			}
