@@ -7,6 +7,7 @@
 package system
 
 import (
+	"os"
 	"syscall"
 )
 
@@ -16,6 +17,15 @@ func CloseDescriptor(fd int) {
 	syscall.Close(fd)
 }
 
+func FileRegularOrLink(filename string) bool {
+	fi, err := os.Stat(filename)
+	if err != nil {
+		return false
+	}
+
+	return fi.Mode().IsRegular() || (fi.Mode()&os.ModeSymlink != 0)
+}
+
 //======================================================================
 // Local Variables:
 // mode: Go
