@@ -22,6 +22,7 @@ import (
 	"path/filepath"
 	"regexp"
 	"runtime"
+	"sort"
 	"strconv"
 	"strings"
 	"sync"
@@ -35,9 +36,12 @@ import (
 	"github.com/gcla/gowid"
 	"github.com/gcla/gowid/gwutil"
 	"github.com/gcla/gowid/vim"
+	"github.com/gcla/gowid/widgets/table"
 	"github.com/gcla/termshark/v2/system"
 	"github.com/gcla/termshark/v2/widgets/resizable"
 	"github.com/gdamore/tcell"
+	"github.com/gdamore/tcell/terminfo"
+	"github.com/gdamore/tcell/terminfo/dynamic"
 	"github.com/mattn/go-isatty"
 	"github.com/pkg/errors"
 	"github.com/shibukawa/configdir"
@@ -97,10 +101,13 @@ var InternalErr = InternalError{}
 //======================================================================
 
 var (
-	UserGuideURL string = "https://termshark.io/userguide"
-	FAQURL       string = "https://termshark.io/faq"
-	BugURL       string = "https://github.com/gcla/termshark/issues/new?assignees=&labels=&template=bug_report.md&title="
-	FeatureURL   string = "https://github.com/gcla/termshark/issues/new?assignees=&labels=&template=feature_request.md&title="
+	UserGuideURL         string = "https://termshark.io/userguide"
+	FAQURL               string = "https://termshark.io/faq"
+	BugURL               string = "https://github.com/gcla/termshark/issues/new?assignees=&labels=&template=bug_report.md&title="
+	FeatureURL           string = "https://github.com/gcla/termshark/issues/new?assignees=&labels=&template=feature_request.md&title="
+	OriginalEnv          []string
+	ShouldSwitchTerminal bool
+	ShouldSwitchBack     bool
 )
 
 //======================================================================
@@ -124,6 +131,14 @@ func DirOfPathCommand(bin string) (string, error) {
 
 //======================================================================
 
+func ReverseStringSlice(s []string) {
+	for i, j := 0, len(s)-1; i < j; i, j = i+1, j-1 {
+		s[i], s[j] = s[j], s[i]
+	}
+}
+
+//======================================================================
+
 // The config is accessed by the main goroutine and pcap loading goroutines. So this
 // is an attempt to prevent warnings with the -race flag (though they are very likely
 // harmless)
@@ -325,6 +340,27 @@ func CacheDir() string {
 // circumstances for a non-existent file, meaning I need to track a directory,
 // and I don't want to be constantly triggered by log file updates.
 func PcapDir() string {
+	var res string
+	// If use-tshark-temp-for-cache is set, use that
+	if ConfBool("main.use-tshark-temp-for-pcap-cache", false) {
+		tmp, err := TsharkSetting("Temp")
+		if err == nil {
+			res = tmp
+		}
+	}
+	// Otherwise try the user's preference
+	if res == "" {
+		res = ConfString("main.pcap-cache-dir", "")
+	}
+	if res == "" {
+		res = DefaultPcapDir()
+	}
+	return res
+}
+
+// DefaultPcapDir returns ~/.cache/pcaps by default. Termshark will check a
+// couple of user settings first before using this.
+func DefaultPcapDir() string {
 	return path.Join(CacheDir(), "pcaps")
 }
 
@@ -771,6 +807,103 @@ func SaveGlobalMarks(m map[rune]GlobalJumpPos) {
 
 //======================================================================
 
+// IPCompare is a unit type that satisfies ICompare, and can be used
+// for numerically comparing IP addresses.
+type IPCompare struct{}
+
+func (s IPCompare) Less(i, j string) bool {
+	x := net.ParseIP(i)
+	y := net.ParseIP(j)
+	if x != nil && y != nil {
+		if len(x) != len(y) {
+			return len(x) < len(y)
+		} else {
+			for i := 0; i < len(x); i++ {
+				switch {
+				case x[i] < y[i]:
+					return true
+				case y[i] < x[i]:
+					return false
+				}
+			}
+			return false
+		}
+	} else if x != nil {
+		return true
+	} else if y != nil {
+		return false
+	} else {
+		return i < j
+	}
+}
+
+var _ table.ICompare = IPCompare{}
+
+//======================================================================
+
+func PrunePcapCache() error {
+	// This is a new option. Best to err on the side of caution and, if not, present
+	// assume the cache can grow indefinitely - in case users are now relying on this
+	// to keep old pcaps around. I don't want to delete any files without the user's
+	// explicit permission.
+	var diskCacheSize int64 = int64(ConfInt("main.disk-cache-size-mb", -1))
+
+	if diskCacheSize == -1 {
+		log.Infof("No pcap disk cache size set. Skipping cache pruning.")
+		return nil
+	}
+
+	// Let user use MB as the most sensible unit of disk size. Convert to
+	// bytes for comparing to file sizes.
+	diskCacheSize = diskCacheSize * 1024 * 1024
+
+	log.Infof("Pruning termshark's pcap disk cache at %s...", PcapDir())
+
+	var totalSize int64
+	var fileInfos []os.FileInfo
+	err := filepath.Walk(PcapDir(),
+		func(path string, info os.FileInfo, err error) error {
+			if err == nil {
+				totalSize += info.Size()
+				fileInfos = append(fileInfos, info)
+			}
+			return nil
+		},
+	)
+	if err != nil {
+		return err
+	}
+
+	sort.Slice(fileInfos, func(i, j int) bool {
+		return fileInfos[i].ModTime().Before(fileInfos[j].ModTime())
+	})
+
+	filesRemoved := 0
+	curCacheSize := totalSize
+	for len(fileInfos) > 0 && curCacheSize > diskCacheSize {
+		err = os.Remove(filepath.Join(PcapDir(), fileInfos[0].Name()))
+		if err != nil {
+			log.Warnf("Could not remove pcap cache file %s while pruning - %v", fileInfos[0].Name(), err)
+		} else {
+			curCacheSize = curCacheSize - fileInfos[0].Size()
+			filesRemoved++
+		}
+		fileInfos = fileInfos[1:]
+	}
+
+	if filesRemoved > 0 {
+		log.Infof("Pruning complete. Removed %d old pcaps. Cache size is now %d MB",
+			filesRemoved, curCacheSize/(1024*1024))
+	} else {
+		log.Infof("Pruning complete. No old pcaps removed. Cache size is %d MB",
+			curCacheSize/(1024*1024))
+	}
+
+	return nil
+}
+
+//======================================================================
+
 var cpuProfileRunning *abool.AtomicBool
 
 func init() {
@@ -868,7 +1001,7 @@ func ConvertArgToTShark(arg string) (string, string, bool) {
 
 var UnexpectedOutput = fmt.Errorf("Unexpected output")
 
-// Use tshark's output, becauses the indices can then be used to select
+// Use tshark's output, because the indices can then be used to select
 // an interface to sniff on, and net.Interfaces returns the interfaces in
 // a different order.
 func Interfaces() (map[int][]string, error) {
@@ -918,6 +1051,33 @@ func interfacesFrom(reader io.Reader) (map[int][]string, error) {
 
 //======================================================================
 
+var foldersRE = regexp.MustCompile(`:\s*`)
+
+// $ env TMPDIR=/foo tshark -G folders Temp
+// Temp:                   /foo
+// Personal configuration: /home/gcla/.config/wireshark
+// Global configuration:   /usr/share/wireshark
+//
+func TsharkSetting(field string) (string, error) {
+	out, err := exec.Command(TSharkBin(), []string{"-G", "folders"}...).Output()
+	if err != nil {
+		return "", err
+	}
+
+	scanner := bufio.NewScanner(strings.NewReader(string(out)))
+	for scanner.Scan() {
+		line := scanner.Text()
+		pieces := foldersRE.Split(line, 2)
+		if len(pieces) == 2 && pieces[0] == field {
+			return pieces[1], nil
+		}
+	}
+
+	return "", fmt.Errorf("Field %s not found in output of tshark -G folders", field)
+}
+
+//======================================================================
+
 func IsTerminal(fd uintptr) bool {
 	return isatty.IsTerminal(fd) || isatty.IsCygwinTerminal(fd)
 }
@@ -1167,6 +1327,19 @@ func FileSizeDifferentTo(filename string, cur int64) (int64, bool) {
 	return newSize, diff
 }
 
+func Does256ColorTermExist() error {
+	return ValidateTerm(fmt.Sprintf("%s-256color", os.Getenv("TERM")))
+}
+
+func ValidateTerm(term string) error {
+	var err error
+	_, err = terminfo.LookupTerminfo(term)
+	if err != nil {
+		_, _, err = dynamic.LoadTerminfo(term)
+	}
+	return err
+}
+
 //======================================================================
 // Local Variables:
 // mode: Go
